use crate::types::{ExMessageGeneric, MessageValue};

pub struct StratumRequest<State> {
    pub(crate) state: State,
    // pub(crate) values: serde_json::map::Map<String, serde_json::Value>,
    pub(crate) values: MessageValue,
}

impl<State> StratumRequest<State> {
    ///  Access application scoped state.
    pub fn state(&self) -> &State {
        &self.state
    }

    pub fn get_ex_message(&self) -> Option<ExMessageGeneric> {
        match &self.values {
            MessageValue::StratumV1(_) => None,
            MessageValue::ExMessage(value) => Some(value.clone()),
        }
    }

    pub fn get_json<T: serde::de::DeserializeOwned>(
        &self,
        name: &str,
    ) -> Result<T, serde_json::Error> {
        match &self.values {
            //@todo switch up these names for clarity too many value
            MessageValue::StratumV1(value) => {
                let value = value
                    .get(name)
                    .ok_or_else(|| serde::de::Error::custom(format!("expected {}", name)))?
                    .clone();

                Ok(serde_json::from_value(value)?)
            }
            MessageValue::ExMessage(_) => Err(serde::de::Error::custom(format!(
                "wrong message type request"
            ))),
        }
    }
}
